// IButtonWorker.java

import java.util.*;
import com.ibutton.*;
import com.ibutton.adapter.*;
import com.ibutton.container.*;

//----------------------------------------------------------------------------
/** FindiButtonsConsole is a console application to view all of the iButtons
 *  on the currently available adapters.
 *
 *  @version    0.00, 1 January 1999
 *  @author     DS
 */	
public class IButtonWorker
{
  static int 	TABLESIZE = 22; // update this number when the table is updated
  
  final String IDTable[] = {
	"000000008AD3C102", "Kingpin",
	"9C00000041177604", "Hobbit",
	"98009000000B3F96", "Oblivion",
	"E9000000004F4B96", "Allen",
	"7200000000529196", "Silicosis",
	"D600000002997C1A", "DilDog",
	"750000000094621A", "Tan",
	"5B009000000F3B96", "Mudge",
	"200000000D0B5916", "Weld Pond",
	"E90000000D008516", "Brian Carrier", 
	"0000000000000000", "Nobody"
  };

	
 /** Constructor
   */
  public IButtonWorker()
  {
  }
  
  // Access and send commands to DS2406 Dual Addressable Switch
  // (controls transistor for door strike solenoid)
  public void dualSwitch (boolean open)
  {
  	DSPortAdapter   adapter = new TINIInternalAdapter();
  	
  	try
  	{
 		// verify there is an adapter detected          
    	if (adapter.adapterDetected())
    	{
    		// clear any previous search restrictions
        	adapter.setSearchAlliButtons();
        	adapter.targetFamily(0x12);
    
    		iButtonContainer12    ibc = (iButtonContainer12)adapter.getFirstiButton();
    	
   			/* A 'TRUE' state is where the PIO pin is conductive to ground 
   			 * A 'FALSE' state is where the PIO is floating. 
   			 */
   			if (!open)
   			{
   			   	ibc.setSwitchState(true);
     			//System.out.println("VaxBar: CLOSED");
     		}
   			else
   			{
   				ibc.setSwitchState(false);
   				//System.out.println("VaxBar: OPEN"); 
   			}
   		}
  
   		// free this port 
    	adapter.endExclusive();
	}
	
    catch(Exception e) 
    {
    };
  }
  
  
  // Perform table look-up of valid ID's, return 1 if valid
  public String checkForValid(String ID)
  {
  	int i;
  	
  	for (i = 0; i < TABLESIZE; i += 2)
  	{
  		if ( ID.compareTo(IDTable[i]) == 0 )
  			return (IDTable[i] + " " + IDTable[i+1]);
  	}
  		
  	return (ID + " Invalid");  
  }
  
   //-------------------------------------------------------------------------
   /** Main for find iButtons test
    */
   public String findiButtons()
   {
      // enumerate through each of the adapter classes
      for(Enumeration adapter_enum = iButtonAccessProvider.enumerateAllAdapters();
                         adapter_enum.hasMoreElements(); )
      {
         // get the next adapter DSPortAdapter
         DSPortAdapter adapter = (DSPortAdapter)adapter_enum.nextElement();

         // get the port names we can use and try to open, test and close each
         for(Enumeration port_name_enum = adapter.getPortNames(); 
                            port_name_enum.hasMoreElements(); ) 
         {
            // get the next packet
            String port_name = (String)port_name_enum.nextElement();

            /* You could check to make sure that it is a valid TINI port here */
            
            try
            {
               // select the port
               adapter.selectPort(port_name);
               
               // verify there is an adapter detected          
               if (adapter.adapterDetected())
               {
                  // clear any previous search restrictions
                  adapter.setSearchAlliButtons();
                  adapter.targetAllFamilies();

                  // enumerate through all the iButtons found
                  for(Enumeration ibutton_enum = adapter.getAlliButtons(); 
                                     ibutton_enum.hasMoreElements(); ) 
                  {
                     // get the next ibutton
                     iButtonContainer ibutton = (iButtonContainer)ibutton_enum.nextElement();
   
   					if (adapter.getAdapterName() == "TINIExternalAdapter")
   					{
   						String ID, Valid;
   						
   						ID = ibutton.getAddressAsString();
   						Valid = checkForValid(ID);
   						return (Valid);
   					}
                  }
               }

               // free this port 
               adapter.endExclusive();
            }
            catch(Exception e) 
            {
            };
         }
      }
      
      return ("");
   }
}

