import java.net.*;
import java.io.*;

public abstract class MemoryReporter
  extends Thread
{
  private static final String NAME = "MemoryReporter";
  private ServerSocket  serverSocket;
  private boolean       testInProgress;
  private long          interval      = 1000;
  private int           portNumber    = 4401;
  int                   freeRAM       = 0;
  byte[]                buffer        = new byte[4];

  /** start a thread that will begin reporting the memory
    * usage via TCP on port <code>portNumber</code> at
    * <code>interval</code> intervals.
    */
  public void startTest(long interval, int portNumber)
  {
    try
    {
      this.portNumber = portNumber;
      serverSocket    = new ServerSocket(portNumber);
      testInProgress  = true;
      this.interval   = interval;
      start();
    }
    catch(Throwable t)
    {
      //System.out.println(toString() + " - startTest failed: " + t.toString());
    }
  }
  /** start a thread that will begin reporting the memory
    * usage via TCP on the default port at
    * <code>interval</code> intervals.
    */
  public void startTest(int interval)
  {
    startTest(interval, portNumber);
  }
  /** start a thread that will begin reporting the memory
    * usage via TCP on the default port at default
    * intervals.
    */
  public void startTest()
  {
    startTest(interval, portNumber);
  }
  /**
    */
  public void run()
  {
    //System.out.println(toString() + " - starting report server");
    Socket        socket = null;
    InputStream   in     = null;
    OutputStream  out    = null;

    try
    {
      socket  = serverSocket.accept();
      in      = socket.getInputStream();
      out     = socket.getOutputStream();

      while(testInProgress)
      {
        freeRAM = com.dalsemi.system.TINIOS.getFreeRAM();
        buffer[0] = (byte)(freeRAM & 0x0ff);
        buffer[1] = (byte)((freeRAM >>> 8)  & 0x0ff);
        buffer[2] = (byte)((freeRAM >>> 16) & 0x0ff);
        buffer[3] = (byte)((freeRAM >>> 24) & 0x0ff);
        out.write(buffer);
        Thread.sleep(interval);
      }
    }
    catch(Throwable t)
    {
      //System.out.println(toString() + " - Aborted report server: " + t.toString());
    }

    try
    {
      in.close();
      out.close();
      socket.close();
      serverSocket.close();
    }
    catch(IOException io)
    {
    }
  }
  /** stop the report server
    */
  public void stopTest()
  {
    testInProgress = false;
  }
  /** return our name
    */
  public String toString()
  {
    return NAME;
  }
}