/*
 * TINIWebServer.java
 */

import java.net.*;
import java.io.*;
import java.util.*;
import com.dalsemi.system.*;

/** This class implements a web server using the HTTPServer class. Logging is implemented using
  * the HTTPServer default logging option. Applications may leave default logging disabled and
  * implement their own logging methods.
  */
public class TINIWebServer
  extends MemoryReporter
{
  WebWorker         webWorker;
  IButtonWorker		iButtonWorker;

  byte[]  copyBuffer;
  String  webRoot,
          webIndex;
  static  Object  lock    = new Object();// lock for file access
  Clock   clock           = new Clock();

  /**  Constructor
    */
  public TINIWebServer()
    throws IOException
  {
     copyBuffer = new byte[8192];
  }
  
  /** Concatenate top and bottom of index page around log.
    */
  public void createPage(String logEvent)
  {
    try
    {
	  clock.getRTC();
      int hour    = clock.getHour();
      int minute  = clock.getMinute();
      int second  = clock.getSecond();
      int month	  = clock.getMonth();
      int date    = clock.getDate();
      int year    = clock.getYear();

      String timeString = new String(month + "/" + date + "/");

	  if(year < 10)
		timeString += "0";
	  timeString += Integer.toString(year);
		
	  timeString += " ";
	  
	  if(hour < 10)
	  	timeString += "0";
	  timeString += Integer.toString(hour);
	  timeString += ":";
 
      if(minute < 10)
        timeString += "0";
      timeString += Integer.toString(minute);
      timeString += ":";

      if(second < 10)
        timeString += "0";
      timeString += Integer.toString(second);
      	
      timeString += " GMT ";
	  timeString += logEvent;
	  timeString += "<BR>";
 	  
      synchronized(lock)
      {
      	// create new index.html
        FileOutputStream  index     = new FileOutputStream(new File(webRoot, webIndex));
        FileInputStream   tempFile  = null;
        
        int bytesRead = (tempFile = new FileInputStream(new File(webRoot,"indextop.html"))).read(copyBuffer);
        index.write(copyBuffer, 0, bytesRead);
        tempFile.close();
        
		// append current log entry
		File indexPage = new File(webRoot+"indexlog.html");
		if (indexPage.length() > 8192)
			indexPage.delete();
		
        FileOutputStream tempIndex = new FileOutputStream((webRoot+"indexlog.html"), true);
        tempIndex.write(timeString.getBytes());
        tempIndex.close();
           
        bytesRead = (tempFile = new FileInputStream(new File(webRoot,"indexlog.html"))).read(copyBuffer);
        index.write(copyBuffer, 0, bytesRead);
        tempFile.close();
      
        bytesRead = (tempFile = new FileInputStream(new File(webRoot,"indexbottom.html"))).read(copyBuffer);
        index.write(copyBuffer, 0, bytesRead);
        tempFile.close();

        index.close();
      }
    }
    catch(Exception e)
    {
      //System.out.println("createPage -"+e.toString());
    }
  }
  
  /** Start the various servers. Then check for iButtons and
    * update the web page.
    */
  public void drive()
  {
  	String logEvent;
        
    iButtonWorker.dualSwitch(false); // door is locked as default
    
    try
    {
      startTest(2000);
      // create the web server
      Thread webServer = new Thread(webWorker);
      webServer.setName("Web Server");

      webRoot   = webWorker.getWebRoot();
      webIndex  = webWorker.getWebPage();

      // start the web server
      webServer.start();

      // periodically probe the 1-wire interface 
      while(true)
      {
      	Thread.sleep(100);
      	
      	logEvent = iButtonWorker.findiButtons();
      	if (logEvent.length() > 0)
      	{
      		// Update logs
      		createPage(logEvent);
      		
      		// If valid, open the door strike
      		if (logEvent.endsWith("Invalid") == false)
      		{
	      		iButtonWorker.dualSwitch(true); // energize door strike 
      			Thread.sleep(5000); // keep open for small time
      			iButtonWorker.dualSwitch(false);
      		}
    	}
      }
    }
    catch(Throwable t)
    {
      //System.out.println("createPageLocal -"+t.toString());
      
      // why kill the server if the exception is not fatal?
      stopTest();
    }
  }
  
  /** Create a TINIWebServer.
    */
  public static void main(String[] args)
  {
    //System.out.println("Starting TINI WebServer version 2.0 ...");

    try
    {
      int currentArg    = 0;

      TINIWebServer tiniWebServer     = new TINIWebServer();
      
      tiniWebServer.webWorker         = new WebWorker(tiniWebServer.lock);
      tiniWebServer.iButtonWorker 	  = new IButtonWorker();

      tiniWebServer.drive();
    }
    catch(Throwable t)
    {
      //System.out.println(t);
      //System.out.println(t.getMessage());      
    }
    finally
    {
      //System.out.println("\nTINIWebserver exiting...");
    }
  }
}
